export class Scrollbars {
    /**
     * The scrollbar DOM element.
     *
     * @type {DOM}
     */
    elements: any;

    /**
     * Selector on elements to apply to.
     *
     * @type {string}
     */
    selector: string = '';

    /**
     * Create a new instance.
     *
     * @param  {string} selector
     */
    constructor(selector: string = '') {
        this.selector = selector;
        this.init();
        this.observe();
    }

    /**
     * Initalizer.
     *
     * @return {void}
     */
    init(): void {
        if (this.selector) {
            this.elements = document.querySelectorAll(this.selector);
            this.elements.forEach((element) => this.create(element));
        }

        this.resize();
    }

    /**
     * Create new scrollbar elements.
     *
     * @param  {HTMLElement} element
     * @return {void}
     */
    create(element: HTMLElement) {
        this.parentClass(element);
        this.height(element);
        this.visible(element);
        this.onScroll(element);
        this.onClick(element);
    }

    /**
     * Destory the scrollbar attributes.
     *
     * @param  {HTMLElement} element
     * @return {void}
     */
    destroy(element: HTMLElement) {
        element.parentElement.classList.remove('scrollbars-container');
        element.classList.remove('scrollbar-hidden');
        element.removeEventListener('scroll');
        element.removeEventListener('click');
        //element.removeEventListener('drag');
        delete element.dataset['scroll'];
    }

    /**
     * Observe the dom for new scrollbar elements.
     *
     * @return {void}
     */
    observe(): void {
        var target = document.body;
        var observer = new MutationObserver(function(mutations) {
            mutations.forEach(function(mutation) {
                if (mutation.type == 'childList') {
                    //this.create();
                }
            });
        });

        observer.observe(target, {
            attributes: true,
            childList: true,
            characterData: true
        });
    }

    /**
     * Resize the scrollbars on window resize event.
     *
     * @return {void}
     */
    resize(): void {
        window.addEventListener('resize', () => {
            this.elements.forEach(element => this.height(element));
        });
    }

    /**
     * Add class to the parent elemeent.
     *
     * @param  {HTMLElement} element
     * @return {void}
     */
    parentClass(element: HTMLElement): void {
        element.parentElement.classList.add('scrollbars-container');
    }

    /**
     * Determine the height of the scrollbar.
     *
     * @param  {HTMLElement} element
     * @return {void}
     */
    position(element: HTMLElement): void {
        let ratio = (element.scrollTop / element.scrollHeight);
        let data_scroll = Math.round(ratio * 1000);

        element.dataset['scroll'] = data_scroll.toString();
    }

    /**
     * Adjust the scroll bar height.
     *
     * @param {HTMLElement} element
     * @return {void}
     */
    height(element: HTMLElement): void {
        if (element) {
            let ratio = element.clientHeight / element.scrollHeight;
            let height = ratio * 100;
            let data_height = Math.floor(height);

            element.dataset['scrollBar'] = data_height.toString();
        }
    }

    /**
     * Determin if the scroll should be visible.
     *
     * @param {HTMLElement} element
     * @return {void}
     */
    visible(element: HTMLElement): void {
        if (element.scrollHeight > element.clientHeight) {
            element.classList.remove('scrollbar-hidden');
        } else {
            element.classList.add('scrollbar-hidden');
        }
    }

    /**
     * Set the right margin of the scrolling element.
     *
     * @param {HTMLElement} element
     * @return {void}
     */
    margin(element: HTMLElement): void {
        element.style.marginRight = '-15px';
    }

    /**
     * On scroll event handler.
     *
     * @param {HTMLElement} element
     * @return {void}
     */
    onScroll(element: HTMLElement): void {
        element.addEventListener('scroll', (event: MouseEvent) => {
            this.position(<HTMLElement>event.target);
            this.height(<HTMLElement>event.target);
        });
    }

    /**
     * On click event handler.
     *
     * @param  {HTMLElement} element
     * @return {void}
     */
    onClick(element: HTMLElement): void {
        element.addEventListener('click', (e: MouseEvent) => {
            if (e.target !== element) return;

            if (e.target['offsetWidth'] - e.x < 30) {
                let click_percentage = Math.round((e.clientY / element.clientHeight) * 100);
                let scroll_top = (element.scrollHeight * click_percentage) / 100;

                element.scrollTop = scroll_top;
            }
        });
    }

    onDrag(element: HTMLElement): void {
        element.addEventListener('click', (e: MouseEvent) => {
            if (e.target !== element) return;

            if (e.target['offsetWidth'] - e.x < 30) {

            }
        });
    }
}
